QToolBar, QToolButton {
    background: transparent;
    margin: 0;
    padding: 0;
}

QToolBar::separator {
    background: #6b6b6b; /* Border */
    width: 1px;
    margin: 6px 3px;
}

QToolButton#qt_toolbar_ext_button {
    qproperty-icon: none;
    margin: 6px 1px;
    background: #eee;
}

QToolButton#qt_toolbar_ext_button:hover {
    background: white;
}

QToolButton {
    padding: 4px 2px;
}

QToolButton:hover, QMenu::item:selected {
    background-color: #333;
}

QStackedWidget, QWidget, QApplication, QDialog,
QLabel, QStatusBar, QTabBar, QMenu,
QDockWidget::title, QHeaderView, QHeaderView::section,
QListWidget, QTreeView, QTreeView::branch {
    background: #222;
    color: white;
}

QTextEdit, QPlainTextEdit, QLineEdit {
    background-color: #373737;
    color: white;
}

QTabBar, QToolBar, QToolButton, QTextEdit, QPlainTextEdit, QLineEdit,
QTabBar::close-button, QTabBar::close-button:hover, QsciScintilla,
QMenu::item, QMenu::item:selected, QTreeView, QDockWidget,
QMainWindow::separator, QScrollBar::handle {
    border: none;
}

QListWidget, QDialog QTextEdit, QDialog QPlainTextEdit, QDialog QLineEdit {
    border: 1px solid #6b6b6b;
}

QTabBar::tab {
    border-top: 1px solid #6b6b6b;   /* Border */
    border-left: 1px solid #6b6b6b;  /* Border */
    border-right: 1px solid #6b6b6b; /* Border */
    border-top-left-radius: 5px;
    border-top-right-radius: 5px;
    color: white;
    padding: 4px 8px;
    margin: 0;
}

QTabBar::tab, QListWidget::item:selected, QListWidget::item:focus, 
QTreeView::item:selected, QTreeView::item:focus {
    background: #474747;  /* Background Tab */
}

QTabBar::tab::hover, QListWidget::item:hover, QTreeView::item:hover {
    background: #5c5c5c; /* Hover */
}

QTabBar::tab::selected {
    background: #6b6b6b;
}

QTabBar::tab QLabel {
    background: transparent;
}

QTabBar::tab QPushButton {
    padding: 3px;
    margin: 0;
    background: transparent;
    border: none;
    min-width: 10px;
    min-height: 10px;
    max-width: 10px;
    max-height: 10px;
}

QTabBar::tab QPushButton:hover {
    background: #c93827;
    border-radius: 3px;
    border: none;
}

QTabWidget::pane {
    border: none;
    border-top: 1px solid #6b6b6b;
    border-bottom: 1px solid #6b6b6b;
}

QTabWidget::tab-bar {
    subcontrol-position: top left;
}

QListWidget::item:selected, QListWidget::item:hover, QListWidget::item:focus,
QTreeView::item:selected, QTreeView::item:hover, QTreeView::item:focus {
    color: white;
    outline: none;
}

QStatusBar QLabel {
    padding: 2px;
    margin: 0;
}

#DeviceSelector QComboBox {
    padding: 4px 25px 4px 3px;
    margin: 2px;
}

#AdministrationLabel:hover, #AdministrationLabel:focus {
    background: #444;
}

QMenu {
    border: 1px solid #6b6b6b;
}

QMenu::item {
    padding: 5px 15px;
}

QMenu::item:disabled {
    color: #333;
}

QMenu::separator {
    border: none;
    border-bottom: 1px solid #333;
    padding: 0px;
    margin: 0px;
}

QCheckBox::indicator {
    border: 1px solid #6b6b6b;
    width: 16px;
    height: 16px;
}

QCheckBox::indicator:checked {
    image: url(images:checked.png);
}

QPushButton {
    background: #474747;            /* Background Tab */
    border: 1px solid #6b6b6b;      /* Border */
    padding: 6px 12px;                /* 1.5 * Tab Padding */
    margin: 2px;
}

QPushButton:focus {
    margin: 1px;
    border: 2px solid #929292;
}

QPushButton:hover {
    background: #5c5c5c;            /* Hover */
}

QPushButton:pressed {
    background: #6b6b6b;            /* Border */
}

QPushButton:disabled {
    background:#222;
}

QAbstractScrollArea::corner {
    border-top: 1px solid #6b6b6b;    /* Border */
    border-left: 1px solid #6b6b6b;   /* Border */
}

QScrollBar, QAbstractScrollArea::corner {
    background: #222;            /* Background */
}

QScrollBar:horizontal {
    height: 21px;
    border-top: 1px solid #6b6b6b; /* Border */
    margin: 0 21px;
}

QScrollBar:vertical {
    width: 21px;
    border-left: 1px solid #6b6b6b; /* Border */
    margin: 21px 0;
}

QScrollBar::handle {
    background: #474747;             /* Background Tab */
    min-width: 21px;
    min-height: 21px;
}

QScrollBar::handle:hover {
    background: #5c5c5c;             /* Hover */
}

QScrollBar::add-line, QScrollBar::sub-line {
    width: 20px;
    height: 20px;
    background: #222;                /* Background */
    border: none;
    subcontrol-origin: margin;
}

QScrollBar::add-line:hover, QScrollBar::sub-line:hover {
    background: #5c5c5c;               /* Hover */
}

QScrollBar::sub-line:horizontal {
    border-top: 1px solid #6b6b6b;    /* Border */
    border-right: 1px solid #6b6b6b;  /* Border */
    subcontrol-position: left;
    image: url(images:scroll-left.svg);
}

QScrollBar::add-line:horizontal, QAbstractScrollArea::corner {
    border-top: 1px solid #6b6b6b;    /* Border */
    border-left: 1px solid #6b6b6b;   /* Border */
    subcontrol-position: right;
}

QScrollBar::add-line:horizontal {
    image: url(images:scroll-right.svg);
}

QScrollBar::sub-line:vertical {
    border-left: 1px solid #6b6b6b;   /* Border */
    border-bottom: 1px solid #6b6b6b; /* Border */
    image: url(images:scroll-up.svg);
}

QScrollBar::add-line:vertical {
    border-left: 1px solid #6b6b6b;   /* Border */
    border-top: 1px solid #6b6b6b;    /* Border */
    image: url(images:scroll-down.svg);
}

QDockWidget::title {
    text-align: left;
    border-bottom: 1px solid #6b6b6b;   /* Border */
    padding: 5px 8px;                     /* Tab Padding */ /* + top/bottom border */
    margin: 0;
}

QDockWidget > QWidget {
    border-bottom: 1px solid #6b6b6b;   /* Border */
}

QMainWindow::separator:horizontal,
QMainWindow::separator:vertical {
    width: 8px;
    background: #424446;
}

QMainWindow::separator:horizontal:hover,
QMainWindow::separator:vertical:hover {
    background: #6b6b6b;
}

QHeaderView::section {
    border-bottom: 1px solid #6b6b6b;   /* Border */
}

QTreeView::item, QListWidget::item {
    padding: 0;
    margin: 0;
}

QTreeView::branch:has-children:open {
    image: url(images:scroll-down.svg);
}

QTreeView::branch:has-children:closed {
    image: url(images:scroll-right.svg);
}

QComboBox {
    padding: 2px;
    border: 1px solid #6b6b6b;
}

QComboBox::drop-down {
    border: 0;
    width: 15px;
    image: unset;
    padding-right: 2px;
}

QComboBox::down-arrow {
    image: url(images:dropdown-arrow-night.svg);
}

QGroupBox {
    border: 1px solid #b4b4b4;
    margin-top: 1ex; /* leave space at the top for the title */
}

QGroupBox::title {
    subcontrol-origin: margin;
    left: 8px;
}
